package edu.washington.cs.radishsim;

import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

public enum EventType {
   INVALID_EVENT(0),
   /** Tracking the Parsec region-of-interest */
   ROI_START(1), ROI_FINISH(2),
   /** When a new thread starts running, so we can setup its resources */
   THREAD_START(3),
   /** When a thread exits, so we can teardown its resources */
   THREAD_FINISH(4),
   /** When a thread is about to block inside the kernel, e.g. when doing a join */
   THREAD_BLOCKED(5),
   /** When a thread returns from blocking inside the kernel, e.g. after having joined */
   THREAD_UNBLOCKED(6), MEMORY_READ(7), MEMORY_WRITE(8), MEMORY_ALLOCATION(9), MEMORY_FREE(10),
   /** Insn counting */
   BASIC_BLOCK(11),
   /** All sync ops are mapped down to HB edges */
   HAPPENS_BEFORE_SOURCE(12), HAPPENS_BEFORE_SINK(13),
   /** Used to verify that frontend can talk to simulator */
   COMMUNICATION_TEST(14);

   private int code;

   private EventType(int code) {
      this.code = code;
   }

   private static final Map<Integer, EventType> lookup = new HashMap<Integer, EventType>();

   static {
      for ( EventType et : EnumSet.allOf(EventType.class) )
         lookup.put(et.asInt(), et);
   }

   public static EventType fromInt(int code) {
      assert lookup.containsKey(code);
      return lookup.get(code);
   }

   public int asInt() {
      return code;
   }

}
