package edu.washington.cs.radishsim;

import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;

@SuppressWarnings( "deprecation" )
public class AsyncSwCheckManagerTests {

   static AsyncSwCheckManager ascm = null;
   static final CpuId p0 = new CpuId(0);
   static final CpuId p1 = new CpuId(1);
   static final int LATENCY = 1000; 
   
   @Before
   public void setUp() {
      ascm = new AsyncSwCheckManager();
   }

   @Test
   public void test_noOverlap() {
      for ( int i = 0; i < 100; i++ ) {
         long wait = ascm.addAsyncCheck(LATENCY, i * LATENCY, p0, false);
         assertEquals(0, wait);
      }
   }
   
   @Test
   public void test_overlap() {
      long wait = ascm.addAsyncCheck(LATENCY, 0, p0, false);
      assertEquals(0, wait);
      
      wait = ascm.addAsyncCheck(LATENCY, 1, p1, false);
      assertEquals(LATENCY-1, wait);
      
      wait = ascm.addAsyncCheck(LATENCY, 2, p0, false);
      assertEquals((2*LATENCY) - 2, wait);
   }
   
   @Test
   public void test_reverseOrder0() {
      long wait = ascm.addAsyncCheck(LATENCY, LATENCY, p1, false);
      assertEquals(0, wait);
      
      // second check can be scheduled before the first
      wait = ascm.addAsyncCheck(LATENCY, 0, p0, false);
      assertEquals(0, wait);
   }
   
   @Test
   public void test_reverseOrder1() {
      long wait = ascm.addAsyncCheck(LATENCY, LATENCY, p0, false);
      assertEquals(0, wait);
      
      // second check overlaps by 5
      wait = ascm.addAsyncCheck(LATENCY, 5, p1, false);
      assertEquals((2*LATENCY)-5, wait);
   }
   
   @Test
   public void test_flush0() {
      long wait = ascm.addAsyncCheck(LATENCY, 0, p0, false);
      assertEquals(0, wait);
      
      wait = ascm.flushAsyncChecks(LATENCY, 100, p1);
      assertEquals(0, wait);
      
      wait = ascm.flushAsyncChecks(LATENCY, 100, p0);
      assertEquals(LATENCY-100, wait);
   }
   
   @Test
   public void test_flush1() {
      long wait = ascm.addAsyncCheck(LATENCY, 0, p0, false);
      assertEquals(0, wait);
      ascm.addAsyncCheck(LATENCY, 100, p0, false);
      ascm.addAsyncCheck(LATENCY, 100, p0, false);
      
      wait = ascm.flushAsyncChecks(LATENCY, 100, p1);
      assertEquals(0, wait);
      
      wait = ascm.flushAsyncChecks(LATENCY, 100, p0);
      assertEquals((3 * LATENCY) - 100, wait);
   }
   
   @Test
   public void test_flush2() {
      long wait = ascm.addAsyncCheck(LATENCY, 0, p0, false);
      assertEquals(0, wait);
      wait = ascm.addAsyncCheck(LATENCY, 100, p1, false);
      assertEquals(LATENCY-100, wait);
      
      wait = ascm.flushAsyncChecks(LATENCY, 500, p1);
      assertEquals((2 * LATENCY) - 500, wait);
      
      wait = ascm.flushAsyncChecks(LATENCY, 500, p0);
      assertEquals(LATENCY - 500, wait);
   }

}
